/*
 * Created on 18.11.2004
 */
package infoscreen;

import java.awt.*;

import javax.swing.*;
import java.awt.event.*;
import java.util.*;

/**
 * @author Christian@Rotzoll.de
 */
public class GuiObjects 
  implements ActionListener,InternalProgramEventListener

{
	private InternalProgramEventManager	eventManager;
	private java.util.Timer				dataMonitorTimer;
	
	// GUI-Basis-Elemente
	JWindow 	mainFrame;
	JPanel  	mainPanel;
	
	// GUI-Elemente fr den Statusbildschirm
	JPanel 		modus_status;
	JButton		stateWindow;
	
	// GUI-Elemente fr den DataInfoBildschirm
	JPanel 		modus_datainfo;
	JTextArea	dataInfoArea;	
	JButton		statusButton;
	
	// GUI-Elemente fr den NewsShowBildschirm
	JPanel		modus_newsshow;
	JPanel      render_panel;
	JButton		changeModeButton;
	
    // Datenelemente
	private String      modus;
	
	
	public GuiObjects (InternalProgramEventManager manager) 
	{
		// Kopiere Referenz auf eventManager 
		this.eventManager = manager; 	
		// Beim eventManager registrieren
		this.eventManager.register(this);
		
		// GUI OBJEKTE ANLEGEN
		
		// Das Hauptfenster anlegen
		this.mainFrame = new JWindow();
		
		// Das Hauptfenster einrichten
		this.mainFrame.setVisible(true);
		this.mainFrame.show();
		
		// ContentPane aus Hauptfenster extrahieren
		// Im mainPanel werden alle anderen Objekte arangiert
		this.mainPanel = (JPanel) mainFrame.getContentPane();
		this.mainPanel.setForeground(new Color(0,0,0,50));
		
		// Init GUI-Panels (StatusPanel)
        this.modus_status = new JPanel();
        this.stateWindow  = new JButton("Init");
        this.stateWindow.setPreferredSize(new Dimension(getScreenWidth()-10,getScreenHeight()-10));
        this.stateWindow.addActionListener(this); 
	    this.modus_status.add(this.stateWindow);
	    this.modus_status.setPreferredSize(new Dimension(getScreenWidth(),getScreenHeight()));	

		// Init GUI-Panels (NewsShowPanel)	    
        this.modus_newsshow = new JPanel();	
        this.modus_newsshow.setOpaque(true);	        
        this.modus_newsshow.setBackground(Color.black);		         
        this.modus_newsshow.setForeground(new Color(0,0,0,50));        
    	this.render_panel   = new JPanel(); 
    	this.render_panel.add(new JButton("... warte auf Daten vom NewsRenderer ..."));
	    this.render_panel.setPreferredSize(new Dimension(getScreenWidth()-2,getScreenHeight()-12));
	    changeModeButton = new JButton("");
	    changeModeButton.setOpaque(true);      
	    changeModeButton.setBackground(Color.black);
	    changeModeButton.setForeground(Color.black);		    
	    changeModeButton.addActionListener(this);
	    changeModeButton.setPreferredSize(new Dimension(getScreenWidth()-2,4));
	    this.modus_newsshow.add(this.render_panel, BorderLayout.PAGE_START);
	    this.modus_newsshow.add(changeModeButton, BorderLayout.PAGE_START);	   
	    this.modus_newsshow.setPreferredSize(new Dimension(getScreenWidth(),getScreenHeight()));
    	
		// Init GUI-Panels (DataInfoPanel)	    
        this.modus_datainfo = new JPanel();
	    this.dataInfoArea  = new JTextArea("dataInfoArea");  
	    this.dataInfoArea.setLineWrap(true);
	    this.dataInfoArea.setPreferredSize(new Dimension(getScreenWidth()-10,(int)(getScreenHeight()*0.95)));
	    this.statusButton  = new JButton("Press to change back.");
	    this.statusButton.addActionListener(this);
	    this.modus_datainfo.add(this.dataInfoArea,BorderLayout.PAGE_START);
	    this.modus_datainfo.add(this.statusButton,BorderLayout.PAGE_END);	
	    this.modus_datainfo.setPreferredSize(new Dimension(getScreenWidth(),getScreenHeight()));
	
	    // DataMonitorTimer init
	    this.dataMonitorTimer = new java.util.Timer();
	    
		// Setzt die GUI in den Statusmodus
		this.setSatusMode();	
		
		// Versuch den Mousecusor zu ndern und aus dem Bild zu setzten
		try {
	      this.mainPanel.getTopLevelAncestor().setCursor(java.awt.Cursor.getPredefinedCursor(java.awt.Cursor.CROSSHAIR_CURSOR));			
		  Robot robot = new Robot();
		  robot.mouseMove(0, 0);
		}
		catch (Exception e)
		{
		  eventManager.infoMeldung("FEHLER GuiObjects@Konstruktor: Der Mauszeiger konnte nicht verschoben werden.");	
		}
	}
	
	////////////////////////////
	// MODUS METHODEN
	///////////////////////////
	// Die folgenden Methoden setzen die das Ausgabefenster in verschiedene 
	// Ausgabemodi. 
	///////////////////////////
	
	////////////////////////////
	// STATUSMODUS 
	// Setzt das Ausgabefenster auf "staus"-modus.
	// Im "Staus"-modus wird immer der letzte Statustext in der Mitte des
	// Bildschirmes ausgegeben.
	public void setSatusMode()
	{	
      // Modus in String speichern
	  modus = "status";		
	  
	  // Alle bisherigen Elemente im mainPanel entfernen
	  this.mainPanel.removeAll(); 	  
	  
	  // Das StatusPanel im MainPanel anzeigen 
	  this.mainPanel.add(this.modus_status, BorderLayout.CENTER);
	  this.mainPanel.repaint();
	  
	  this.mainFrame.pack();
	  this.mainFrame.setSize(this.getScreenWidth(),this.getScreenHeight());		  
	}
	
	////////////////////////////
	// DATAMONITORMODUS 
	// Setzt das Ausgabefenster auf "datainfo"-modus.
	// Im "datainfo"-modus werden alle relevanten Objekte in regelmigen
	// Intervallen ber ihren Zustand und ihre Daten Auskunft zu geben.
	public void setDataInfoMode()
	{
	  // Modus in String speichern
	  modus = "datainfo";	
		
	  // Alle bisherigen Elemente im mainPanel entfernen
	  this.mainPanel.removeAll(); 
	  	    
	  // Das DataInfoPanel im MainPanel anzeigen
	  this.mainPanel.add(this.modus_datainfo);
	  this.mainPanel.repaint();
	  
	  this.updateDataInfoArea();
	  
	  this.mainFrame.pack();
	  this.mainFrame.setSize(this.getScreenWidth(),this.getScreenHeight());	
	  
	  // DataMonitorTimer starten
	  this.dataMonitorTimer.schedule(new MonitorTask(), 10000);
	}
	
    // Wird durch den Timer gesteuert
    class MonitorTask extends TimerTask {
        public void run() {
          printStatus("Datamonitor update");
          updateDataInfoArea();
          if(modus.equals("datainfo"))
          {
      	    dataMonitorTimer.schedule(new MonitorTask(), 10000);  
          }  
        }
    }
	
	// Neuen Statustext setzten
	public void printStatus(String text)
	{
	  // Im Statuspanel updaten	
	  this.stateWindow.setText(text);
	  this.stateWindow.repaint();

	  // Im DataInfoPanel updaten
	  this.statusButton.setText(text);
	  this.statusButton.repaint();

	}
	
	public void updateDataInfoArea()
	{
	  Date date = new Date();
	  this.dataInfoArea.setText("############ DATAMONITOR vom " + date.toGMTString());
	  eventManager.requestDataInfoUpdate();
	}
	
	////////////////////////////
	// NEWSSHOWMODUS 
	// Setzt das Ausgabefenster auf "newsshow"-modus.
	// Im "newsshow"-modus wird das Panel ausgegeben, dass der NewsRender liefert.
	public void setNewsShowMode()
	{	
      // Modus in String speichern
	  modus = "newsshow";		
	  
	  // Alle bisherigen Elemente im mainPanel entfernen
	  this.mainPanel.removeAll(); 	  
	  
	  // Das NewsShowPanel im MainPanel anzeigen 
	  this.mainPanel.setOpaque(true);
	  this.mainPanel.add(this.modus_newsshow, BorderLayout.CENTER);
	  this.mainPanel.repaint();
	  
	  this.mainFrame.pack();
	  this.mainFrame.setSize(this.getScreenWidth(),this.getScreenHeight());		  
	}	
	
	public void setNewsShowPanel(JPanel newPanel)
	{
      this.render_panel = newPanel;
      this.render_panel.setOpaque(true);      
      this.render_panel.setBackground(Color.black);
      
      this.render_panel.setPreferredSize(new Dimension(getScreenWidth()-2,getScreenHeight()-12));
      this.modus_newsshow.removeAll();    
      this.modus_newsshow.add(this.render_panel, BorderLayout.PAGE_START);     
	  this.modus_newsshow.add(changeModeButton, BorderLayout.PAGE_START);    
      
      this.mainPanel.setOpaque(true);      
      this.mainPanel.setBackground(Color.black);	  
	  this.mainPanel.repaint();
	  
	  this.mainFrame.pack();
	  this.mainFrame.setSize(this.getScreenWidth(),this.getScreenHeight());	      
	}
		
	public void actionPerformed (ActionEvent ae)
	{
	  System.out.println("Changing GUI Mode");	
	  
	  if ( this.modus.equals("status") )
	  	{this.setDataInfoMode();}
	  else if (this.modus.equals("datainfo"))
	    {this.setNewsShowMode();}
	  else if (this.modus.equals("newsshow"))
	    {this.setDataInfoMode();}
	}
	
	public void internalProgramEventHappend(InternalProgramEvent ipe)
	{
	  // hier die ankommenden Events auswerten			
      String eventName = ipe.getName();	
      
	  if (eventName.equals("fehlermeldung"))
	  {
	  	// Fehlermeldung auf GUI ausgeben
	  	this.printStatus(ipe.getData());	
	  }
	  else if (eventName.equals("infomeldung"))
	  {
	  	// Infomeldung auf GUI ausgeben
	  	this.printStatus(ipe.getData());	
	  }
	  else if (eventName.equals("dataInfoUpdate"))
	  {
	  	// DataInfo auf GUI ausgeben
	  	this.dataInfoArea.append("\n\n"+ipe.getData());	
	  }		
	  else if (eventName.equals("requestNewsShowModus"))
	  {
	  	// Wenn im StatusModus, dann in den NewsShowModus wechseln
	  	if ( this.modus.equals("status") ) 
		{
	  	  this.setNewsShowMode();	
		}
	  }			  
	}  
	
    //	 Returns the Height of Desktop
	public int getScreenHeight() {
	    java.applet.Applet applet = new java.applet.Applet();
	    java.awt.Toolkit toolkit = applet.getToolkit();
	    Dimension Screen = toolkit.getScreenSize();
	    return (int) Screen.getHeight();
	  }

	// Returns the Width of Desktop
	public int getScreenWidth() {
	    java.applet.Applet applet = new java.applet.Applet();
	    java.awt.Toolkit toolkit = applet.getToolkit();
	    Dimension Screen = toolkit.getScreenSize();
	    return (int) Screen.getWidth();
	  }
}
