/*
 * Created on 18.11.2004
 */
package infoscreen;

//import infoscreen.NewsSheduler.ShedulerTask;

import java.util.*;
import java.net.*;
import java.io.*;

/**
 * @author Christian Desktop
 */
public class NetworkInterface 
implements InternalProgramEventListener
{
	private Timer 						timer;
	private ServerThread				server;						
	
	private NetworkUtils				networkUtils;
	
    private NewsContainer				newsContainer;
	private InternalProgramEventManager	eventManager;
	
	private Vector						NewsData;
	private Date					    lastNewsUpdate;
	
    // Speichert die Befehle vom InfoScreenControl-Script
    private Vector 						instructions;
	
	public NetworkInterface (NewsContainer con, InternalProgramEventManager ipem)
	{
	  // Referenzen kopieren
	  this.eventManager 	= ipem; 
	  this.newsContainer   	= con;
	  
	  // Beim eventManager registrieren
	  this.eventManager.register(this);	
	  
	  // Networkutils-Objekt anlegen (bietet notwendige Netzfunktionen)
	  this.networkUtils = new NetworkUtils(this.eventManager);
	  
	  // NewsData init
	  NewsData = new Vector(1);
	  
	  // Lastupdate init
	  this.lastNewsUpdate = new Date(0);
	
	  // Timer fr NetworkTask starten
	  this.timer = new Timer();	  
	  if (this.eventManager.getModus().equals("remote"))
	  {
	  	// Leeren Befehlsspeicher initialisieren
	  	instructions = new Vector();
	  	
	  	// Es wird der ServerThread gestartet, der auf Befehle vom ControlScript warten
        server = new ServerThread("InfoscreenServerThread");
        server.start();	  	
	  	
	    // Es wird der NetworkTask fr den REMOTE-Modus gestartet	
	  	timer.schedule(new RemoteNetworkTask() , 10);
	  }
	  else
	  {
	    // Es wird der NetworkTask fr den STANDALONE-Modus gestartet
	  	timer.schedule(new StandaloneNetworkTask() , 10);	  	
	  }
	}
	
	public void internalProgramEventHappend(InternalProgramEvent ipe)
	{
	  // hier die ankommenden Events auswerten	
      String eventName = ipe.getName();	
      
	  if (eventName.equals("dataInfoUpdateRequest"))
	  {
	  	// DatenBericht dieses Objektes zusammenstellen und verschicken
	  	eventManager.dataInfoUpdate("NETWORKINTERFACE: \nLetzte Aktualisierung des Newsbestandes: "+this.lastNewsUpdate.toGMTString());
	  }				
	} 	
	
    // Wird durch den Timer gesteuert
    class RemoteNetworkTask extends TimerTask 
	{
    	   	
      public void run() 
      {
        	
      	 String controlScript = eventManager.getUrl()+"/infoscreenCom.php?screenname="+eventManager.getInfoscreenName(); 
      	
         // Prfe, ob noch Befehle vorliegen 
    	 if (instructions.size()==0)
    	 {
           // Neue Befehle vom SteuerScript holen -------------

      	   eventManager.infoMeldung("Lade Befehle vom InfoscreenControlCenter ");
      	   NewsData = networkUtils.getWWWpage(controlScript+"&action=getOrder");
      	  
    	   if (NewsData.size() > 0)
  		   {
  		     // Befehle verarbeiten	
			 eventManager.infoMeldung("Verbindung OK. Befehle werden verarbeitet.");
			  
			 // Prfen, ob die Daten gltig sind
			 if (!((String) NewsData.get(0)).equals("Infoscreen Befehle:"))
			 {
			   // Es wurde ein Fehler entdeckt	
			   eventManager.fehlerMeldung("FEHLER in Daten vom PHP-Script : " + ((String) NewsData.get(1)) );	
			    
			   // Nchster Versuch in 2 Minuten
			   timer.schedule(new RemoteNetworkTask(), 120*1000);
			  }
			  else
			  { 
			    // Die Daten sind OK -> Befehle gespeichert	
			  	NewsData.removeElementAt(0);
		        instructions = NewsData;
		        
		        // Dieser Task startet sich nun neu, um die Befehle abzuarbeiten
	  	        timer.schedule(new RemoteNetworkTask(), 1000); 		        
  		      }    			   			             
  		    }	
  		    else
  		    {
  		      // Keine Verbindung bzw.Daten erhalten 
      	      eventManager.infoMeldung("Keine Verbindung. Erneuter Versuch in 30 Sekunden."); 
      	  
  		      // Nchster Versuch in 30 Sekunden
  	          timer.schedule(new RemoteNetworkTask(), 30*1000);    			
  		    } 			
    	  }
    	  else
    	  {
            // noch vorliegende Befehle ausfhren -------------
    	  	
    	  	// vorderste Befehlszeile holen   	  	
    	  	String instructionRow = (String) instructions.get(0);
    	  	instructions.removeElementAt(0);
    	  	
    	  	String[] parameter  = instructionRow.split("XXtrennerXX");
    	  	
    	  	if ((parameter!=null) && (parameter[0]!=null))
    	  	{
    	  	  String instruction = parameter[0];
    	  	  
    	  	  System.out.println("BEFEHL: '"+instruction+"'");
    	  	  
    	  	  if (instruction.equals("WARTE"))
    	  	  {
    	  	     if (newsContainer.getNumberOfNewsObjects()>1)
    	  	     {
       	  	       eventManager.infoMeldung("Befehl: Warte 15 Minuten");
                   // Dieser Task startet sich neu, um andere Befehle abzuarbeiten    	  	     	
	               timer.schedule(new RemoteNetworkTask(), 900*1000);  
    	  	     }
    	  	     else
    	  	     {
    	  	       eventManager.infoMeldung("Befehl: Warte 2 Minuten");
                   // Dieser Task startet sich neu, um andere Befehle abzuarbeiten    	  	     	
	               timer.schedule(new RemoteNetworkTask(), 120*1000);      	  	     	
    	  	     }
    	  	  }
    	  	  else if (instruction.equals("REGISTRATIONINFO"))
    	  	  {
    	  	     eventManager.infoMeldung("OK. Der Infoscreen hat sich beim Kontroll-Script registriert. Er muss jetzt unter '"+eventManager.getUrl()+"' einer Gruppe zugeordnet werden.");    	  	    
	             // Dieser Task startet sich in 2 Minuten neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 120*1000);   
    	  	  }  
    	  	  else if (instruction.equals("LADE_ALLE_DATEIEN_UEBER_KONTROLSCRIPT"))
    	  	  {
    	  	     eventManager.setProxyMode(true);   	  	    
	             // Dieser Task startet sich in 2 Minuten neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 100);   
    	  	  }      	  
    	  	  else if (instruction.equals("LADE_ALLE_DATEIEN_DIREKT_AUS_DEM_NETZ"))
    	  	  {
    	  	     eventManager.setProxyMode(false);   	  	    
	             // Dieser Task startet sich in 2 Minuten neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 100);   
    	  	  }         	  	  
    	  	  else if (instruction.equals("TEST_PROXY"))
    	  	  {
  	  	         Vector dats = networkUtils.getWWWpage(controlScript+"&action=getFile&url="+eventManager.getUrl()+"/proxytest.txt");
  	  	         if (((String)dats.get(0)).equals("OK"))
  	  	         {
  	  	           System.out.println("Proxytest: ERFOLGREICH!");
  	               // Dieser Task startet sich in 2 Minuten neu, um andere Befehle abzuarbeiten
    	           timer.schedule(new RemoteNetworkTask(), 100);     	  	           
  	  	         }
  	  	         else
  	  	         {
  	  	          eventManager.infoMeldung("Bitte berprfen Sie die Proxyeinstellungen fr diesen Screen im Control-Script."); 
                  for (Enumeration lines = dats.elements(); lines.hasMoreElements();)
                  {
                    System.out.println((String)lines.nextElement());	
                  }
 	              // Dieser Task startet sich in 2 Minuten neu, um andere Befehle abzuarbeiten
   	              timer.schedule(new RemoteNetworkTask(), 5000);   
  	  	         }
    	  	  }      	      	  	  
    	  	  else if (instruction.equals("NEWSDATAUPDATE_START"))
    	  	  {
    	  	     eventManager.stopSheduler();
    	  	     eventManager.infoMeldung("Empfange neue NewsDaten.");     	  	     
    	  	     newsContainer.setAllDirty();
	             // Dieser Task startet sich gleich neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 500);   
    	  	  }    	   
    	  	  else if ((instruction.equals("SHOW_IMPRESSUM")))
    	  	  {
    	  	     String name 	    = parameter[1];
    	  	  	 String mail  		= parameter[2];
    	  	     String post    	= parameter[3];
    	  	     
    	  	     System.out.println(name+"/"+mail+"/"+post);
    	  	     
    	  	     newsContainer.createImpressum(name,mail,post);  	  	 
    	      	 
	             // Dieser Task startet sich gleich neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 200);   
    	  	  }   				  
    	  	  else if ((instruction.equals("KANAL_PARAMETER")) && (parameter.length == 4))
    	  	  {
    	  	     String channel_name 	= parameter[1];
    	  	  	 int importance  		= (new Integer(parameter[2])).intValue();
    	  	     int daylimit    		= (new Integer(parameter[3])).intValue();
    	  	     newsContainer.setFeedImportParameter(channel_name,importance,daylimit);  	  	 
    	      	 
	             // Dieser Task startet sich gleich neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 200);   
    	  	  }   	  	  
    	  	  else if ((instruction.equals("LADE_KANAL")) && (parameter.length == 2))
    	  	  {
    	  	  	 String channel_url 	= parameter[1];
    	  	  	 if (eventManager.getProxyMode())
    	  	  	 {
    	  	  	   channel_url = controlScript + "&action=getFile&url=" + channel_url;	
    	  	  	 }
    	  	  	 System.out.println("Lade Feed mit der Adresse: '"+channel_url+"'");
    	  	     newsContainer.importFeedFromURLString(channel_url);  	  	  	 
    	      	 
	             // Dieser Task startet sich gleich neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 200);   
    	  	  }  
    	  	  else if (instruction.equals("NEWSDATAUPDATE_ENDE"))
    	  	  {
    	  	     newsContainer.deleteAllDirtyNews();  
    			 // Lastupdate setzten
    			 lastNewsUpdate = new Date();
    				
    	  	     if (newsContainer.getNumberOfNewsObjects()>0)
    	  	     {
    	  	       eventManager.startSheduler();
    			   eventManager.requestNewsShowModus();  
    	  	     }
    	  	     else
    	  	     { 
       	  	       eventManager.infoMeldung("Keine Nachrichten erhalten.");      	  	     	
    	  	     }

	             // Dieser Task startet sich gleich neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 5000);   
    	  	  }    	        	  	  
    	  	  else
    	  	  {  // BEFEHL: unbekannt
        	  	 System.out.println("Hinweis: Befehlszeile unbekannt: "+instructionRow);
	             // Dieser Task startet sich neu, um andere Befehle abzuarbeiten
  	             timer.schedule(new RemoteNetworkTask(), 1000);     	  	  	
    	  	  }
    	  	}
    	  	else
    	  	{
    	  	  System.out.println("Hinweis: Befehlszeile ungltig");
	          // Dieser Task startet sich neu, um andere Befehle abzuarbeiten
  	          timer.schedule(this, 1000); 	    	  		
    	  	}       	  
    	  }	     	
       } 	
	}
	
    // Wird durch den Timer gesteuert
    class StandaloneNetworkTask extends TimerTask 
	{
        public void run() 
        {
        	
        	// Newsdataupdate
    	  	eventManager.infoMeldung("Verbinde mit " + eventManager.getUrl());
		  	
			// Feed-Url dem NewsContainer zum Import bergeben
			newsContainer.setAllDirty();
			newsContainer.importFeedFromURLString(eventManager.getUrl()); 			
			newsContainer.deleteAllDirtyNews();
	  	    eventManager.startSheduler();			
			    
			// Lastupdate setzten
			lastNewsUpdate = new Date();
			    
			// GUI signalisieren, dass Nachrichten geladen wurden
			eventManager.requestNewsShowModus();
			    
			// nchstes Datenupdate in 15 Minuten
			timer.schedule(new StandaloneNetworkTask (), 900*1000);			    			   			             
           }       	
      } 

    // Serverthread der auf eine Anfrage vom ContolScript wartet
    public class ServerThread extends Thread {
    	
        public ServerThread(String str) {
            super(str);
        }
        
        public void run() {
          System.out.println("Der SERVERTHREAD wurde gestartet.");
          
          
          try {
          
          // Auf Server auf Port XXXX einrichten
          ServerSocket server = new ServerSocket( 3143 );
          
	          while ( true )
	          {
	          	// Warten, auf Anfrage
	            PrintWriter out = null;
	            BufferedReader in = null;
	            Socket client = server.accept();
	            
	            // Ein- und Ausgabekanle einrichten
	            out = new PrintWriter(client.getOutputStream(), true);
	            in = new BufferedReader(new InputStreamReader(
	                                        client.getInputStream()));
	            
	            // Aktion
	            System.out.println("Serverthread Verbindung eingegangen --- " + client.getRemoteSocketAddress());
	            
	            // Anfrage auslesen
	            String line = in.readLine(); 
	            System.out.println("Empfangen: '"+line+"'");
	            
	            // Das ControlSript testet nur die Erreichbarkeit
	            if (line.equals("Hello Infoscreen"))
	            {
		          // Antwort geben	
	              String response = "Hi Controlscript :-)";
		          out.println(response);	  
		          System.out.println("Gesendet: '"+response+"'");		          
	            }
	            else if (line.equals("update Channelmix"))
	            {
	              // Aktuellen Timer lschen
	              timer.cancel();
	              // Neuen Timer mit sofortiger Ausfhrung starten
	              timer = new Timer();
	        	  timer.schedule(new RemoteNetworkTask() , 10);	  
	        	  
		          // Antwort geben	
	              String response = "done";
		          out.println(response);	  
		          System.out.println("Gesendet: '"+response+"'");		        	  
	            }
	            else if (line.equals("Newsflash"))
	            {
		          // Parameter auslesen
		          String dauer = in.readLine(); 
		          System.out.println("Parameter DAUER: '"+dauer+"'");
		          String text = in.readLine(); 
		          System.out.println("Parameter TEXT: '"+text+"'");		          
	        	  
		          // Newsflash-Event
		          eventManager.requestNewsflash(text,dauer);
		          
		          // Antwort geben	
	              String response = "done";
		          out.println(response);	  
		          System.out.println("Gesendet: '"+response+"'");		        	  
	            }	            
	            else

	            out.flush();
	            
	            client.close();
	            System.out.println("Serverthread Verbindung beendet ---");
	          }
          
          }
          catch (IOException ioe)
		  {
            eventManager.fehlerMeldung("@Serverthread : Der Port 3143 scheint besetzt zu sein.");	
		  }
        }
    }
}
