/*
 * Created on 18.11.2004
 */
package infoscreen;

import java.util.*;

/**
 * @author Christian Desktop
 */
public class NewsSheduler 
implements InternalProgramEventListener
{
	private Timer 						timer;
	private boolean						timeraktiv;	
	
	// Referenz auf das aktuelle NewsObject aus dem NewsContainer
	private NewsObject					actualNewsObject;	
	
	// Anzeigestatus des aktuellen Newsobjetes
	private int					     	stateOfNewsObject;
	final int NO_NEWSOBJECT     = -3; // kein NewsObject vorhanden
	final int END_OF_NEWSOBJECT = -2; // das NewsObject wurde vollstndig angezeigt
	final int HEADLINE          = -1; // momentan wird die Headline angezeigt	
	//                                  >= 0  // aktuelles NewsComponent des NewsObjektes
	
    private NewsContainer				newsContainer;
	private NewsRenderer 				newsRenderer;
	private InternalProgramEventManager	eventManager;
	
	public NewsSheduler (NewsContainer con, NewsRenderer ren, InternalProgramEventManager ipem)
	{
	  // Referenzen kopieren
	  this.eventManager 	= ipem; 
	  this.newsContainer   	= con;
	  this.newsRenderer		= ren;
	  
	  // Beim eventManager registrieren
	  this.eventManager.register(this);	  
	  
	  this.actualNewsObject  = null;
	  this.stateOfNewsObject = this.NO_NEWSOBJECT;
	  
	  // Timer erstellen (noch nicht starten)
      this.timeraktiv = false;
	}
	
	public void startTimer()
	{
	  // Der Sheduler startet (wieder)
	  if (!this.timeraktiv)
	  {
		eventManager.infoMeldung("Sheduler ist aktiv");	  	
	    this.timeraktiv = true;
	    timer = new Timer();
	    timer.schedule(new ShedulerTask(), 10);
	  }
	}
	
	public void stopTimer()
	{
	  // Der Sheduler startet (wieder)
	  if (this.timeraktiv)
	  {		
	    this.timeraktiv = false;	
	    timer.cancel();
	    eventManager.infoMeldung("Sheduler ist inaktiv");	
	  }
	}	
	
	public void pauseTimer(int Dauer)
	{
	  // Der Sheduler startet (wieder)
	  if (this.timeraktiv)
	  {		
	    this.timeraktiv = false;	
	    timer.cancel();
	    timer = new Timer();
	    timer.schedule(new ShedulerTask(), Dauer);	 
	    this.timeraktiv = true;	    
	    eventManager.infoMeldung("Sheduler pausiert");	
	  }
	}	

    // Wird durch den Timer gesteuert
    class ShedulerTask extends TimerTask {
        public void run() {
        	if (timeraktiv)
        	{
    		  if ((stateOfNewsObject == NO_NEWSOBJECT) || (stateOfNewsObject == END_OF_NEWSOBJECT))
    		  {	
    		    // Hole die nchste News aus dem NewsContainer

    		  	  actualNewsObject = newsContainer.getNextNewsObjekt();    		  		

    		    if (actualNewsObject!=null)
    		    {
    		      stateOfNewsObject = HEADLINE;
                  eventManager.infoMeldung("Scheduling next News: "+actualNewsObject.getHeadline());
                
                  // Die Headline ausgeben/rendern ...
                  int Duration = newsRenderer.renderHeadline(actualNewsObject);
    		  	  // ... danach wieder den ShedulerTask aufrufen      		  	  
                  timer.schedule(new ShedulerTask(), Duration); 
    		    }
    		    else
    		    {
                  eventManager.infoMeldung("Keine News im Newscontainer. Scheduler inaktiv.");
                  // Scheduler legt sich schlafen indem der Timer nicht neu gesetzt wird
                  // NewsContainer weckt bei Eingang von Nachrichten
    		    }
    		  }
    		  else
    		  {
    		  	// Die NewsComponents des NewsObjectes ausgeben
    		  	if ((stateOfNewsObject+1) < actualNewsObject.getSize())
    		  	{
    		      // Nchstes NewsComponent ausgeben/rendern ...		
    		  	  int Duration = newsRenderer.renderNewsComponent(actualNewsObject,++stateOfNewsObject);      		  	  
    		  	  // ... danach wieder den ShedulerTask aufrufen      		  	  
                  timer.schedule(new ShedulerTask(), Duration);        		  	  
    		  	}
    		  	else
    		  	{
    		  	  // Vermerkt das Ende des NewsObjects	
    		  	  stateOfNewsObject = END_OF_NEWSOBJECT;
    		  	  
    		  	  // ZwischenAnimation ausgeben und ...
    		  	  int Duration = newsRenderer.renderBetweenAnimation();
    		  	  // ... danach wieder den ShedulerTask aufrufen
                  timer.schedule(new ShedulerTask(), Duration);   		  	  
    		  	}
    		  }
        	}
        }
    }   
	
	
	public void internalProgramEventHappend(InternalProgramEvent ipe)
	{
	  // hier die ankommenden Events auswerten	
      String eventName = ipe.getName();	
      
	  if (eventName.equals("startSheduler"))
	  {
	  	// Timer starten
	  	this.startTimer();
	  }	
	  else if (eventName.equals("stopSheduler"))
	  {
	  	// Timer stoppen
	  	this.stopTimer();
	  }	      
	  else if (eventName.equals("newsflash"))
	  {
	  	// Timer fr die Dauer des Newsflashes aussetzten
	    String DauerStr = ipe.getData().substring(0,ipe.getData().indexOf(":"));
	    String NewsStr  = ipe.getData().substring(ipe.getData().indexOf(":")+1);
	    try {
	      Integer Dauer   = new Integer(DauerStr);
	      this.pauseTimer(Dauer.intValue()*60*1000);	 
	      newsRenderer.rederNewsflash(NewsStr);
	    } catch (Exception e) {System.out.println("Newsflash abgebrochen");}
	  }	    	  
	  else if (eventName.equals("dataInfoUpdateRequest"))
	  {
	  	// DatenBericht dieses Objektes zusammenstellen und verschicken
	  	String report = "NEWSSHEDULER: \n";
	  	
	  	if (this.timeraktiv)
	  	{ report += "Timer ist aktiv \n";}
	  	else
	  	{ report += "Timer ist inaktiv \n";}
	  	
	  	if (this.actualNewsObject==null)
	  	{ report += "Aktuelle Nachricht: -keine- ";}
	  	else
	  	{ report += "Aktuelle Nachricht: "+this.actualNewsObject.getHeadline();}
	  	             

	  	if      (this.stateOfNewsObject == this.END_OF_NEWSOBJECT)
	  	{ report += " \nAktuelles Untersegment: Zwischenanimation zum nchsten NewsObjekt";}
	  	else if (this.stateOfNewsObject == this.HEADLINE)
	  	{ report += " \nAktuelles Untersegment: berschriftstafel";}	
	  	else if (this.stateOfNewsObject >= 0)
	  	{ report += " \nAktuelles Untersegment: "+(this.stateOfNewsObject+1)+"/"+actualNewsObject.getSize();}	  	
	  	
	  	eventManager.dataInfoUpdate(report);
	  }			  
	} 
}
